import React, { useCallback, useEffect, useState } from "react"
import { log } from "../logging/logger"
import { DrawerScreenProps } from "@react-navigation/drawer"
import { Box, Button, ChevronDownIcon, ChevronUpIcon, Flex, HStack, MinusIcon, Popover, Switch, Text, VStack, useToast } from "native-base"
import { SortOptions } from "../types/general"
import { Note } from "../types/note"
import NotesListView from "../components/notes/NotesListView"
import { useDispatch, useSelector } from "react-redux"
import { AppDispatch, RootState } from "../stores/store"
import LoadingBox from "../components/loading/LoadingBox"
import { createNote, getAllNotes } from "../stores/actions/notesThunks"
import { RootStackParamList } from "./Navigation"
import { ErrorToast } from "../components/toast/ErrorToast"
import { InfoToast } from "../components/toast/InfoToast"

import { Dimensions } from "react-native";

const NotesViewPage = ({ navigation }: DrawerScreenProps<RootStackParamList, 'Notes'>) => {

  const { notes, notesLoading, requestPending, triggerRefresh } = useSelector((state: RootState) => state.noteViewState);

  const dispatch = useDispatch<AppDispatch>();

  const [isSortOpen, setIsSortOpen] = useState(false);

  const [myCommentsCheck, setMyCommentsCheck] = useState(false);
  const [generalCommentsCheck, setGeneralCommentsCheck] = useState(true);

  const lastError = useSelector((state: RootState) => state.noteViewState.lastError)
  const toast = useToast();

  useEffect(() => {
      if (lastError) {
          toast.closeAll()
          toast.show({
              render: ({
                  id
              }) => {
                  return <ErrorToast headerText={"Error"} text={lastError} onClose={() => toast.close(id)} />;
              },
              duration: 3000
          });
      }
  }, [lastError])

  const [sortSettings, setSortSettings] = useState({
    items: SortOptions.None,
    date: SortOptions.None
  });

  const handleSortChanged = useCallback((key: "items" | "date", sortSettings: any) => {
    let currentState = sortSettings[key];
    if (currentState == SortOptions.None) {
      setSortSettings({
        ...sortSettings,
        [key]: SortOptions.Asc
      })
    }
    else if (currentState == SortOptions.Asc) {
      setSortSettings({
        ...sortSettings,
        [key]: SortOptions.Desc
      })
    }
    else {
      setSortSettings({
        ...sortSettings,
        [key]: SortOptions.None
      })
    }
  }, [setSortSettings]);

  const getSortIcon = (sortOption: SortOptions, size: string) => {
    switch (sortOption) {
      case SortOptions.Asc:
        return <ChevronUpIcon size={size} />;
      case SortOptions.Desc:
        return <ChevronDownIcon size={size} />;
      default:
        return <MinusIcon size={size} />;
    }
  };

  // trigger refresh on triggerRefresh increment
  useEffect(() => {
    log.debug("NotesViewPage", "useEffect", "getNotes");
    getNotes();
  }, [triggerRefresh])

  // general comments check changed
  useEffect(() => {
    if (!notesLoading) {
      log.debug("NotesViewPage", "useEffect", "getNotes");
      getNotes();
    }
  }, [generalCommentsCheck, myCommentsCheck])


  const getNotes = useCallback(() => {
    dispatch(getAllNotes({
      myComments: myCommentsCheck,
      generalComments: generalCommentsCheck,
      sortOptions: sortSettings
    }));
  }, [myCommentsCheck, generalCommentsCheck, sortSettings]);


  const handleCreateComment = useCallback((newComment: string, replyingTo: { messageId: string; userName: string } | null) => {
    console.log(newComment, replyingTo)

    if (!requestPending) {
      // creating new comment
      toast.show({
        render: ({
            id
        }) => {
            return <InfoToast  text={"Adding note"} onClose={() => toast.close(id)} />;
        },
        duration: 2000
    });
      if (replyingTo != null) {
        dispatch(createNote({
          newNote: { note: newComment, reply_to: replyingTo.messageId } as Note
        }))
      }
      else {
        dispatch(createNote({
          newNote: { note: newComment } as Note
        }))
      }
    }
  }, [dispatch, createNote]);



  return (
    <Box marginLeft={2.5} marginRight={2.5} marginTop={2.5}>
      <VStack>
        <HStack height={75} justifyContent="space-between">
          <Box justifyContent={"center"}>
            <Popover // @ts-ignore
              trigger={triggerProps => {
                return <Button colorScheme="primary" {...triggerProps} onPress={() => setIsSortOpen(true)}>
                  Sort Options
                </Button>;
              }} isOpen={isSortOpen} onClose={() => setIsSortOpen(!isSortOpen)}>
              <Popover.Content w="48">
                <Popover.Arrow />
                <Popover.CloseButton onPress={() => setIsSortOpen(false)} />
                <Popover.Header>Sort Options</Popover.Header>
                <Popover.Body>
                  <VStack>
                    <Button size="md" variant="outline" onPress={() => handleSortChanged("items", sortSettings)}
                      rightIcon={getSortIcon(sortSettings.items, "sm")}
                    >
                      Items</Button>
                    <Button size="md" variant="outline" marginTop={"5%"} onPress={() => handleSortChanged("date", sortSettings)}
                      rightIcon={getSortIcon(sortSettings.date, "sm")}>
                      Date</Button>
                  </VStack>
                </Popover.Body>
                <Popover.Footer justifyContent="flex-end">
                  <Button.Group space={2}>
                    <Button colorScheme="coolGray" variant="ghost" onPress={() => setIsSortOpen(false)}>
                      Cancel
                    </Button>
                    <Button colorScheme="success" onPress={() => { setIsSortOpen(false); getNotes() }}>
                      Save
                    </Button>
                  </Button.Group>
                </Popover.Footer>
              </Popover.Content>
            </Popover>
          </Box>

          <VStack space={0.5}>
            <Flex direction="row" alignItems="center" justify="flex-end" height={35}>
              <Text>My comments</Text>
              <Switch isChecked={myCommentsCheck} onValueChange={() => { if (!notesLoading) setMyCommentsCheck(!myCommentsCheck) }} size="md" />
            </Flex>
            <Flex direction="row" alignItems="center" justify="flex-end" height={35}>
              <Text>General comments</Text>
              <Switch isChecked={generalCommentsCheck} onValueChange={() => { if (!notesLoading) setGeneralCommentsCheck(!generalCommentsCheck) }} size="md" />
            </Flex>
          </VStack>
        </HStack>
        {
          notesLoading ? (
            <LoadingBox text="Loading notes" />
          )
            :
            (<NotesListView height={Dimensions.get('window').height - 70 - 160} notes={notes} navigation={navigation} handleCreateComment={handleCreateComment} requestPending={requestPending} />)
        }


      </VStack>
    </Box>
  );
}

export default NotesViewPage;
